@@ -27,21 +27,6 @@ itemsOptionalColumns = {
     "DurationOrigineel": FloatType()
 }
 
-linesAllColumns = [
-    ("DienstId", IntegerType()),
-    ("FacturatieMaand", StringType()),
-    ("VerkooprelatieId", IntegerType()),
-    ("ItemCode", StringType()),
-    ("Aantal", FloatType()),
-    ("AfwijkendePrijs", FloatType()),
-    ("DatumVanOrigineel", StringType()),
-    ("DatumTotOrigineel", StringType()),
-    ("AantalOrigineel", FloatType()),
-    ("DurationOrigineel", FloatType()),
-    ("HasItems", BooleanType()),
-    ("Id", StringType())
-]
-
 itemsAllColumns = [
     ("DienstId", IntegerType()),
     ("Source", StringType()),
@@ -74,6 +59,11 @@ def validateItemsSchema(items):
 def writeItems(items):
     validateItemsSchema(items)
 
+    items = items.withColumn("Id", F.expr("uuid()"))
+    items = items.withColumn("LineId", F.lit(None).cast(StringType()))
+
+    items = writeLines(items)
+
     table = "items"
     tablePath = f"{tdsbrondata.tablesRootPath}/{table}"
 
@@ -85,13 +75,9 @@ def writeItems(items):
     else:
         items.write.format("delta").mode("overwrite").option("overwriteSchema", "true").save(tablePath)
 
-    writeLines(items)
-
 def writeLines(items):
-
     table = "lines"
     tablePath = f"{tdsbrondata.tablesRootPath}/{table}"
-
     facturatieMaand = items.select("FacturatieMaand").head()["FacturatieMaand"]
 
     existingitemsOptionalColumns = [col for col in itemsOptionalColumns.keys() if col in items.columns]
@@ -128,12 +114,28 @@ def writeLines(items):
         ).otherwise(False)
     ).drop("SurrogateKeys").drop("Source")
 
+    aggregated = aggregated.withColumn("Id", F.expr("uuid()"))
+
+    itemsToAggregate = itemsToAggregate.join(
+        aggregated.select(
+            "DienstId", "FacturatieMaand", "VerkooprelatieId", "ItemCode", "Id"
+        ),
+        on=["DienstId", "FacturatieMaand", "VerkooprelatieId", "ItemCode"],
+        how="left"
+    ).withColumnRenamed("Id", "LineId")
+
     itemsToPreserve = itemsToPreserve.withColumn(
         "HasItems",
         F.when(F.col("SurrogateKey").isNotNull(), True).otherwise(False)
-    )
+    ).join(
+        aggregated.select(
+            "DienstId", "FacturatieMaand", "VerkooprelatieId", "ItemCode", "Id"
+        ),
+        on=["DienstId", "FacturatieMaand", "VerkooprelatieId", "ItemCode"],
+        how="left"
+    ).withColumnRenamed("Id", "LineId")
 
-    aggregated = aggregated.select([c for c in itemsToPreserve.columns if c not in ["Source", "SurrogateKey"]])
+    aggregated = aggregated.select([c for c in itemsToPreserve.columns if c not in ["Source", "SurrogateKey", "LineId"]])
 
     itemsToPreserve = itemsToPreserve.drop("Source", "SurrogateKey")
 
@@ -146,6 +148,38 @@ def writeLines(items):
     else:
         lines.write.format("delta").mode("overwrite").option("overwriteSchema", "true").save(tablePath)
 
+    return itemsToAggregate.unionByName(itemsToPreserve)
+
+def mergeItems(lists, facMonth):
+
+    items = []
+
+    for schema in lists:
+        
+        df = tdsbrondata.utils.getCurrentData(
+            workspaceName='Tosch-Facturatie',
+            lakehouseName='S_Items',
+            schemaName=schema,
+            tableName='items',
+            usesScd=False
+        )
+
+        df = df.filter(F.col("FacturatieMaand") == facMonth)
+        
+        for colName, colType in itemsOptionalColumns.items():
+            if colName not in df.columns:
+                df = df.withColumn(colName, F.lit(None).cast(colType))
+            if colType == FloatType():
+                df = df.withColumn(colName, F.round(F.col(colName).cast(FloatType()), 2))
+            else:
+                df = df.withColumn(colName, F.col(colName).cast(colType))
+        
+        items.append(df)
+
+    dfItems = reduce(lambda df1, df2: df1.unionByName(df2), items)
+
+    return dfItems
+
 def mergeLines(lists, facMonth):
 
     lines = []
@@ -175,32 +209,3 @@ def mergeLines(lists, facMonth):
 
     return dfLines
 
-def mergeItems(lists, facMonth):
-
-    items = []
-
-    for schema in lists:
-        
-        df = tdsbrondata.utils.getCurrentData(
-            workspaceName='Tosch-Facturatie',
-            lakehouseName='S_Items',
-            schemaName=schema,
-            tableName='items',
-            usesScd=False
-        )
-
-        df = df.filter(F.col("FacturatieMaand") == facMonth)
-        
-        for colName, colType in itemsAllColumns:
-            if colName not in df.columns:
-                df = df.withColumn(colName, F.lit(None).cast(colType))
-            if colType == FloatType():
-                df = df.withColumn(colName, F.round(F.col(colName).cast(FloatType()), 2))
-            else:
-                df = df.withColumn(colName, F.col(colName).cast(colType))
-        
-        items.append(df)
-
-    dfItems = reduce(lambda df1, df2: df1.unionByName(df2), items)
-
-    return dfItems
\ No newline at end of file
